/*
 * sensor.c
 *
 * CCS C compiler
 *
 * Sensor unit. Reads frequency from each color channel on color sensor,
 * Processes two pots for CCT and brightness, displays results on LCD
 *
 *      Authors: Sichao Wang, Jered Greenspan
 *      Copyright 2012
 */

#include <16F887.h>
#device *= 16
#fuses HS, NOWDT, NOPROTECT, NOLVP, PUT
#fuses NOBROWNOUT, CPD, NODEBUG, NOWRT
#use delay(clock=20MHz,osc)
#use rs232(baud=1000, xmit=PIN_C6, rcv=PIN_C7, bits=8, parity=N)
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "Button.c"
#include "lcd_new.c"

#define FACTOR 4000000L
char gc_capture_flag = 0;
char overflowflag = 0;
int8 numOverflows = 0;

int16 r,g,b;
int8 cct, bright;
int16 f = 0;

char buf[50] = "";

#INT_TIMER1
void overflow1(void) {
	numOverflows++;
}

#int_ccp2
void ccp2_isr(void) {
	if (gc_capture_flag == 0) {
		gc_capture_flag = 1;
		set_timer1(0);
	} else if (gc_capture_flag == 1) {
		f = CCP_2;
		gc_capture_flag = 2;
	}
}

void print_lcd_16(int16 out) {
	int8 i = 0;
	sprintf(buf, "%lu", out);
	while (buf[i] != '\0') {
		lcd_putc(buf[i]);
		i++;
	}
}
void print_lcd_8(int8 out) {
	int8 i = 0;
	sprintf(buf, "%u", out);
	while (buf[i] != '\0') {
		lcd_putc(buf[i]);
		i++;
	}
}
int16 sensorValue(int16 f) {
	return FACTOR / ((int32) numOverflows * 0xFFFF + (int32) f);
}

void main() {
	int8 i = 0;
	BOOLEAN light_on = 1;
	int8 but = 0;
	int8 pack_count = 0;

	lcd_init();
	lcd_putc("\f");

	setup_adc_ports(sAN5 | sAN6);
	setup_adc(ADC_CLOCK_INTERNAL);
	setup_timer_1(T1_INTERNAL);
	setup_ccp2(CCP_CAPTURE_DIV_16);

	clear_interrupt(INT_CCP2);
	enable_interrupts(INT_CCP2);
	enable_interrupts(INT_TIMER1);
	disable_interrupts(GLOBAL);

	// set frequency scale to 100%
	output_high(PIN_A6);
	output_high(PIN_C0);

	delay_ms(500);

	output_low(PIN_C4); //backlight
	output_high(PIN_D6);

	while (true) {
		delay_ms(10);

		// read button
		if (button(PIN_E2, 0, 50, 10, button_down, 1)) {
			BOOLEAN light_on = 1;
		}
		if (button_down) {
			output_high(PIN_D7);
			light_on = !light_on;
		} else {
			output_low(PIN_D7);
		}

		clear_interrupt(INT_CCP2);
		clear_interrupt(INT_TIMER1);

		gc_capture_flag = 0;
		overflowflag = 0;
		numOverflows = 0;
		enable_interrupts(GLOBAL);
		while (gc_capture_flag != 2);
		disable_interrupts(GLOBAL);
		clear_interrupt(INT_CCP2);
		clear_interrupt(INT_TIMER1);

		// get R
		output_low(PIN_D1);
		output_low(PIN_D0);
		gc_capture_flag = 0;
		overflowflag = 0;
		numOverflows = 0;
		enable_interrupts(GLOBAL);
		while (gc_capture_flag != 2);
		disable_interrupts(GLOBAL);
		r = sensorValue(f);
		clear_interrupt(INT_CCP2);
		clear_interrupt(INT_TIMER1);

		// get G
		output_high(PIN_D1);
		output_high(PIN_D0);
		gc_capture_flag = 0;
		overflowflag = 0;
		numOverflows = 0;
		enable_interrupts(GLOBAL);
		while (gc_capture_flag != 2);
		disable_interrupts(GLOBAL);
		g = sensorValue(f);
		clear_interrupt(INT_CCP2);
		clear_interrupt(INT_TIMER1);

		// get B
		output_low(PIN_D0);
		output_high(PIN_D1);
		gc_capture_flag = 0;
		overflowflag = 0;
		numOverflows = 0;
		enable_interrupts(GLOBAL);
		while (gc_capture_flag != 2);
		disable_interrupts(GLOBAL);
		b = sensorValue(f);

		// read potentiometers
		set_adc_channel(6);
		delay_us(10);
		// cct ranges from 2 to 9
		cct = 11 - ((read_adc() >> 5) + 2);

		set_adc_channel(5);
		delay_us(10);
		// brightness ranges from 0 to 100
		bright = 100 - ((int32) read_adc() * 0.3921569);

		clear_interrupt(INT_CCP2);
		clear_interrupt(INT_TIMER1);

		putc(1);
		putc(0);
		putc(1);
		putc(0);
		putc(1);
		putc(0);
		putc(1);
		putc(0);
		putc(1);
		// Send header
		putc(255);
		putc(255);
		// Send data
		putc(r % 256);
		putc(r >> 8);
		putc(g % 256);
		putc(g >> 8);
		putc(b % 256);
		putc(b >> 8);
		putc(cct);
		putc(0);
		putc(bright);
		putc(light_on);

		// Write to LCD

		if (light_on) {
			output_high(PIN_D6); // indicator on
			output_high(PIN_C4); // backlight on
			lcd_putc("\fCCT:");
			print_lcd_8(cct);
			lcd_putc("000K ");
			print_lcd_8(bright);
			lcd_putc("%\n");
			print_lcd_16(r);
			lcd_putc(" ");
			print_lcd_16(g);
			lcd_putc(" ");
			print_lcd_16(b);
		} else {
			output_low(PIN_D6);
			output_low(PIN_C4);
		}
	}
}
